/*
 * bs.c : ubN\[g̃eXg
 *
 *         Copyright (C) 2004 Makoto Hiroi
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#define TRUE  1
#define FALSE 0

#define BUFF_SIZE   0x100000
#define CODE_SIZE   0x100
#define ISORT_NUM   10
#define HEADER_SIZE 4
#define MERGE       1
#define MQUICK      2
#define TWO0        3
#define TWO1        4
#define TWO2        5
#define TWO3        6
#define TWO31       7
#define TWO32       8

typedef unsigned char  Uchar;

int  eflag = FALSE;
int  dflag = FALSE;

char input_name[FILENAME_MAX];
char output_name[FILENAME_MAX];
char *file_header ="BS__";

/* obt@` */
Uchar buffer[BUFF_SIZE * 2];
Uchar work_buffer[BUFF_SIZE];
Uchar *index_table[BUFF_SIZE];
Uchar **work_table;

/* \[gf[^ */
int sort_count;

/* [N̈擾 */
void *get_buffer( int size )
{
  void *buff = malloc( size );
  if( buff == NULL ){
    fprintf( stderr, "Out of Memory\n" ); exit( 1 );
  }
  return buff;
}

/***** \[g ******/

/* P}\[g */
void insert_sort( int low, int high, int depth, int size )
{
  int i, j, len = size - depth;
  for( i = low + 1; i <= high ; i++ ){
    Uchar *tmp = index_table[i];
    for( j = i - 1;
         j >= low && memcmp( tmp + depth, index_table[j] + depth, len ) < 0;
         j-- ){
      index_table[j + 1] = index_table[j];
    }
    index_table[j + 1] = tmp;
  }
}

/* }[W\[g */
void merge_sort( int low, int high, int depth, int size )
{
  if( (high - low + 1) <= ISORT_NUM ){
    insert_sort( low, high, depth, size );
  } else {
    int middle = (low + high) / 2;
    int i, p, j, k;
    merge_sort( low, middle, depth, size );
    merge_sort( middle + 1, high, depth, size );
    p = 0;
    i = low;
    while( i <= middle ) work_table[p++] = index_table[i++];
    i = middle + 1;
    j = 0;
    k = low;
    while( i <= high && j < p ){
      if( memcmp(work_table[j] + depth, index_table[i] + depth, size - depth) <= 0 ){
        index_table[k++] = work_table[j++];
      } else {
        index_table[k++] = index_table[i++];
      }
    }
    while( j < p ) index_table[k++] = work_table[j++];
  }
}

/* }N֐ */
#define MIN( a, b )   ((a) < (b) ? (a) : (b))

/* }N */
#define SWAP( i, j ) {   \
  char *_tmp = index_table[(i)];\
  index_table[(i)] = index_table[(j)]; \
  index_table[(j)] = _tmp;      \
}

/* ̑I */
int select_pivot( int low, int high, int depth )
{
  int a = *(index_table[low] + depth);
  int b = *(index_table[(low + high) / 2] + depth);
  int c = *(index_table[high] + depth);
  int d;
  if( a > b ){
    d = a; a = b; b = d;
  }
  /* ł a < b */
  if( b > c ){
    b = c;
    /* ł b < c, Ō a Ɣr */
    if( a > b ) b = a;
  }
  return b;
}

/* Multikey Quicksort */
void mquick_sort( int low, int high, int depth, int size )
{
  while( 1 ){
    if( high - low + 1 <= ISORT_NUM ){
      insert_sort( low, high, depth, size );
      break;
    } else {
      int pivot, i, j, k, l, m1, m2;
      /* ̑I */
      pivot = select_pivot( low, high, depth );
      /* Ԃ 4  */
      i = m1 = low;
      j = m2 = high;
      for(;;){
        while( i <= j ){
          k = *(index_table[i] + depth) - pivot;
          if( k > 0 ) break;
          if( k == 0 ){
            /* Ɠl[ֈړ */
            SWAP( i, m1 );
            m1++;
          }
          i++;
        }
        while( i <= j ){
          k = *(index_table[j] + depth) - pivot;
          if( k < 0 ) break;
          if( k == 0 ){
            /* Ɠl[ֈړ */
            SWAP( j, m2 );
            m2--;
          }
          j--;
        }
        if( i > j ) break;
        SWAP( i, j );
        i++;
        j--;
      }
      /* [ = 𒆉Ɉړ */
      k = MIN( m1 - low, i - m1 );
      for( l = 0; l < k; l++ ) SWAP( low + l, j - l );
      m1 = low + (i - m1);
      /* E[ = 𒆉Ɉړ */
      k = MIN( high - m2, m2 - j );
      for( l = 0; l < k; l++ ) SWAP( i + l, high - l);
      m2 = high - (m2 - j) + 1;
      /* < \[g */
      if( low < m1 ) mquick_sort( low, m1 - 1, depth, size );
      /* > \[g */
      if( m2 <= high ) mquick_sort( m2, high, depth, size );
      /* = \[g */
      if( m1 >= m2 || depth + 1 == size ) break;
      low = m1;
      high = m2 - 1;
      depth++;
    }
  }
}

/***** z\[gp ******/
int count[CODE_SIZE * CODE_SIZE];
int count_sum[CODE_SIZE * CODE_SIZE + 1];

#define GETCODE( n ) ((buffer[(n)] << 8) + buffer[(n) + 1])

/* 擪 2 Lŕz\[g */
void dist_sort( int size )
{
  int i;
  /* 擪 2 ŕz\[g */
  for( i = 0; i < CODE_SIZE * CODE_SIZE; i++ ) count[i] = 0;
  for( i = 0; i < size; i++ ) count[ GETCODE( i ) ]++;
  count_sum[0] = 0;
  for( i = 1; i <= CODE_SIZE * CODE_SIZE; i++ ){
    count_sum[i] = count[i - 1] + count_sum[i - 1];
  }
  for( i = 1; i < CODE_SIZE * CODE_SIZE; i++ ) count[i] += count[i - 1];
  for( i = size - 1; i >= 0; i-- ){
    int c = GETCODE( i );
    index_table[ --count[c] ] = buffer + i;
  }
}

/* z\[g + }[W\[g */
void sort_merge( int size )
{
  int i;
  /* [N̈̊m */
  work_table = get_buffer( sizeof( Uchar * ) * BUFF_SIZE / 2 );
  /* ԂƂɃ\[g */
  for( i = 0; i < CODE_SIZE * CODE_SIZE; i++ ){
    int low = count_sum[i];
    int high = count_sum[i + 1];
    if( high - low > 1 ){
      sort_count += high - low;
      merge_sort( low, high - 1, 2, size );
    }
  }
  free( work_table );
}

/* z\[g + }`L[NCbN\[g */
void sort_mquick( int size )
{
  int i;
  /* ԂƂɃ\[g */
  for( i = 0; i < CODE_SIZE * CODE_SIZE; i++ ){
    int low = count_sum[i];
    int high = count_sum[i + 1];
    if( high - low > 1 ){
      sort_count += high - low;
      mquick_sort( low, high - 1, 2, size );
    }
  }
}

/* iK\[g@ (ratio-1) + }[W\[g */
void sort_two0( int size )
{
  int i, j;
  /* [N̈̊m */
  work_table = get_buffer( sizeof( Uchar * ) * BUFF_SIZE / 2 );
  for( i = 0; i < CODE_SIZE; i++ ){
    for( j = i; j < CODE_SIZE; j++ ){
      /* Type B \[g */
      int low  = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      if( high - low > 1 ){
        sort_count += high - low;
        merge_sort( low, high - 1, 2, size );
      }
    }
  }
  /* Type A Zbg */
  for( i = 0; i < size; i++ ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) > *ptr ){
      index_table[ count_sum[(*(ptr - 1) << 8) + *ptr]++ ] = ptr - 1;
    }
  }
  free( work_table );
}

/* iK\[g@ (ratio-1) */
void sort_two1( int size )
{
  int i, j;
  for( i = 0; i < CODE_SIZE; i++ ){
    for( j = i; j < CODE_SIZE; j++ ){
      /* Type B \[g */
      int low  = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      if( high - low > 1 ){
        sort_count += high - low;
        mquick_sort( low, high - 1, 2, size );
      }
    }
  }
  /* Type A Zbg */
  for( i = 0; i < size; i++ ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) > *ptr ){
      index_table[ count_sum[(*(ptr - 1) << 8) + *ptr]++ ] = ptr - 1;
    }
  }
}

/* iK\[g@ (ratio-2) */
void sort_two2( int size )
{
  int i, j, end;
  for( i = 0; i < CODE_SIZE; i++ ){
    for( j = i; j < CODE_SIZE; j++ ){
      int n, low  = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      /* Type B 𕪗 */
      for( n = low; n < high; n++ ){
        Uchar *ptr = index_table[n];
        if( *ptr > *(ptr + 2) ){
          index_table[n] = index_table[low];
          index_table[low++] = ptr;
        }
      }
      if( high - low > 1 ){
        sort_count += high - low;
        mquick_sort( low, high - 1, 2, size );
      }
    }
  }
  /* Type A Zbg */
  for( i = 0; i < size; i++ ){
    /* ratio-1 */
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) > *ptr ){
      index_table[ count_sum[(*(ptr - 1) << 8) + *ptr]++ ] = ptr - 1;
    }
    /* ratio-2 */
    ptr = index_table[i];
    if( ptr < buffer + 2 ) ptr += size;
    if( *(ptr - 2) > *ptr && *(ptr - 2) <= *(ptr - 1) ){
      index_table[ count_sum[(*(ptr - 2) << 8) + *(ptr - 1)]++ ] = ptr - 2;
    }
  }
}

/***** O@ *****/

/* Type A, B Zbg */
void set_type_ab( int sym, int size )
{
  int i, start[CODE_SIZE], end[CODE_SIZE];
  /* 2 Ԗڂ̋L sym ̋Ԃ߂ */
  for( i = sym; i < CODE_SIZE; i++ ){
    start[i] = count_sum[(i << 8) + sym];
    end[i]   = count_sum[(i << 8) + sym + 1] - 1;
  }
  /* O */
  for( i = count_sum[sym << 8]; i < start[sym]; i++ ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) >= *ptr ){
      index_table[ start[*(ptr - 1)]++ ] = ptr - 1;
    }
  }
  /* 납O */
  for( i = count_sum[(sym + 1) << 8] - 1; i > end[sym]; i-- ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) >= *ptr ){
      index_table[ end[*(ptr - 1)]-- ] = ptr - 1;
    }
  }
}

/* O@ */
void sort_two3( int size )
{
  int i, j;
  for( i = 0; i < CODE_SIZE; i++ ){
    /* type C \[g */
    for( j = i + 1; j < CODE_SIZE; j++ ){
      int low = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      if( high - low > 1 ){
        sort_count += high - low;
        mquick_sort( low, high - 1, 2, size );
      }
    }
    /* type A, B Zbg */
    set_type_ab( i, size );
  }
}

/***** O@ : Ȃ Type \[g */

/* Type B, C Zbg */
void set_type_bc( int sym, int size )
{
  int i, start[CODE_SIZE], end[CODE_SIZE];
  /* 2 Ԗڂ̋L sym ̋Ԃ߂ */
  for( i = 0; i <= sym; i++ ){
    start[i] = count_sum[(i << 8) + sym];
    end[i]   = count_sum[(i << 8) + sym + 1] - 1;
  }
  /* O */
  for( i = count_sum[sym << 8]; i < start[sym]; i++ ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) <= *ptr ){
      index_table[ start[ *(ptr - 1) ]++ ] = ptr - 1;
    }
  }
  /* 납O */
  for( i = count_sum[(sym + 1) << 8] - 1; i > end[sym]; i-- ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) ptr += size;
    if( *(ptr - 1) <= *ptr ){
      index_table[ end[ *(ptr - 1) ]-- ] = ptr - 1;
    }
  }
}

/* Type A \[g */
void sort_two3_a( int size )
{
  int i, j;
  for( i = CODE_SIZE - 1; i >= 0; i-- ){
    /* type A \[g */
    for( j = 0; j < i; j++ ){
      int low = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      if( high - low > 1 ){
        sort_count += high - low;
        mquick_sort( low, high - 1, 2, size );
      }
    }
    /* Type B, C Zbg */
    set_type_bc( i, size );
  }
}

void sort_two31( int size )
{
  int i, a, c;
  /* type A, C ̌JEg */
  for( i = 0, a = 0, c = 0; i < CODE_SIZE; i++ ){
    a += count_sum[(i << 8) + i] - count_sum[i << 8];
    c += count_sum[(i + 1) << 8] - count_sum[(i << 8) + i + 1];
  }
  /* Ȃ Type \[g */
  if( a > c ){
    sort_two3( size );    /* Type C \[g */
  } else {
    sort_two3_a( size );  /* Type A \[g */
  }
}

/***** O@ : ratio-2 *****/
int start2[CODE_SIZE * CODE_SIZE + 1];

/* type A \[g */
void sort_type_a2( int size )
{
  int i, j;
  for( i = CODE_SIZE - 1; i >= 0; i-- ){
    /* type A (Si > Si+1) 
       type D (Si >= Si+ 2) Type E (Si < Si+2 ) ɕă\[g */
    for( j = 0; j < i; j++ ){
      int m, n, low = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      start2[(i << 8) + j] = high - 1;        /* rate-2 p */
      for( m = n = low; n < high; n++ ){
        Uchar *ptr = index_table[n];
        if( *ptr >= *(ptr + 2) ){
          index_table[n] = index_table[m];
          index_table[m++] = ptr;
        }
      }
      if( m - low > 1 ){
        sort_count += m - low;
        mquick_sort( low, m - 1, 2, size );
      }
    }
  }
}

/* type B, C ɃZbg */
void set_type_bc2( int size )
{
  int i, start[CODE_SIZE], end[CODE_SIZE];
  for( i = CODE_SIZE - 1; i >= 0; i-- ){
    int j, wp;
    for( j = 0; j <= i; j++ ){
      start[j] = count_sum[(j << 8) + i];
      end[j]   = count_sum[(j << 8) + i + 1] - 1;
    }
    /* O */
    for( wp = 0, j = count_sum[i << 8]; j < start[i]; j++ ){
      /* ratio-1 */
      Uchar *ptr = index_table[j];
      if( ptr == buffer ) ptr += size;
      if( *(ptr - 1) <= *ptr ){
        index_table[ start[ *(ptr - 1) ]++ ] = ptr - 1;
      }
      /* ratio-2 */
      ptr = index_table[j];
      if( ptr < buffer + 2 ) ptr += size;
      if( *(ptr - 2) < *ptr && *(ptr - 2) > *(ptr - 1) ){
        /* work_table ɑޔ */
        work_table[wp++] = ptr - 2;
      }
    }
    /* 납O */
    for( j = count_sum[(i + 1) << 8] - 1; j > end[i]; j-- ){
      Uchar *ptr = index_table[j];
      if( ptr == buffer ) ptr += size;
      if( *(ptr - 1) <= *ptr ){
        index_table[ end[ *(ptr - 1) ]-- ] = ptr - 1;
      }
      /* 2 O`FbN */
      ptr = index_table[j];
      if( ptr < buffer + 2 ) ptr += size;
      if( *(ptr - 2) < *ptr && *(ptr - 2) > *(ptr - 1) ){
        index_table[ start2[(*(ptr - 2) << 8) + *(ptr - 1)]-- ] = ptr - 2;
      }
    }
    /* work_table Rs[ */
    while( wp > 0 ){
      Uchar *ptr = work_table[--wp];
      index_table[ start2[(*ptr << 8) + *(ptr + 1)]-- ] = ptr;
    }
  }
}

/* Type C \[g */
void sort_type_c2( int size )
{
  int i, j;
  for( i = 0; i < CODE_SIZE; i++ ){
    /* type C (Si < Si+1) 
       type D (Si > Si+2)  type E (Si <= Si+2) ɕă\[g */
    for( j = i + 1; j < CODE_SIZE; j++ ){
      int m, n, low = count_sum[(i << 8) + j];
      int high = count_sum[(i << 8) + j + 1];
      start2[(i << 8) + j] = low;       /* rate-2 p */
      for( m = n = low; n < high; n++ ){
        Uchar *ptr = index_table[n];
        if( *ptr > *(ptr + 2) ){
          index_table[n] = index_table[m];
          index_table[m++] = ptr;
        }
      }
      if( high - m > 1 ){
        sort_count += high - m;
        mquick_sort( m, high - 1, 2, size );
      }
    }
  }
}

/* type A, B Zbg */
void set_type_ab2( int size )
{
  int i, start[CODE_SIZE], end[CODE_SIZE];
  for( i = 0; i < CODE_SIZE; i++ ){
    int j, wp;
    for( j = i; j < CODE_SIZE; j++ ){
      start[j] = count_sum[(j << 8) + i];
      end[j]   = count_sum[(j << 8) + i + 1] - 1;
    }
    /* O */
    for( j = count_sum[i << 8]; j < start[i]; j++ ){
      /* ratio-1 */
      Uchar *ptr = index_table[j];
      if( ptr == buffer ) ptr += size;
      if( *(ptr - 1) >= *ptr ){
        index_table[ start[ *(ptr - 1) ]++ ] = ptr - 1;
      }
      /* ratio-2 */
      ptr = index_table[j];
      if( ptr < buffer + 2 ) ptr += size;
      if( *(ptr - 2) > *ptr && *(ptr - 2) < *(ptr - 1) ){
        index_table[ start2[(*(ptr - 2) << 8) + *(ptr - 1)]++ ] = ptr - 2;
      }
    }
    /* 납O */
    for( wp = 0, j = count_sum[(i + 1) << 8] - 1; j > end[i]; j-- ){
      Uchar *ptr = index_table[j];
      if( ptr == buffer ) ptr += size;
      if( *(ptr - 1) >= *ptr ){
        index_table[ end[ *(ptr - 1) ]-- ] = ptr - 1;
      }
      /* ratio-2 */
      ptr = index_table[j];
      if( ptr < buffer + 2 ) ptr += size;
      if( *(ptr - 2) > *ptr && *(ptr - 2) < *(ptr - 1) ){
        /* work_table ɑޔ */
        work_table[wp++] = ptr - 2;
      }
    }
    /* work_table Rs[ */
    while( wp > 0 ){
      Uchar *ptr = work_table[--wp];
      index_table[ start2[(*ptr << 8) + *(ptr + 1)]++ ] = ptr;
    }
  }
}

/* O@ : ratio-2 */
void sort_two32( int size )
{
  int i, a, c, max;
  /* type A, C ̌JEg */
  for( i = 0, a = 0, c = 0, max = 0; i < CODE_SIZE; i++ ){
    int n = count_sum[(i << 8) + i] - count_sum[i << 8];
    if( max < n ) max = n;
    a += n;
    n = count_sum[(i + 1) << 8] - count_sum[(i << 8) + i + 1];
    if( max < n ) max = n;
    c += n;
  }
  /* work_table ̈̃Zbg */
  work_table = get_buffer( sizeof( Uchar * ) * max );
  /* Ȃ type \[g */
  if( a > c ){
    sort_type_c2( size );
    set_type_ab2( size );
  } else {
    sort_type_a2( size );
    set_type_bc2( size );
  }
  free( work_table );
}

/*
 * ubN\[g 
 *
 * Input buffer : buffer, Output buffer : work_buffer
 */
int blocksort_encode( int size )
{
  int i, top;
  /* Rs[ */
  memcpy( buffer + size, buffer, size );
  /* \[g */
  dist_sort( size );
  switch( eflag ){
  case MERGE:
    sort_merge( size ); break;
  case MQUICK:
    sort_mquick( size ); break;
  case TWO0:
    sort_two0( size ); break;
  case TWO1:
    sort_two1( size ); break;
  case TWO2:
    sort_two2( size ); break;
  case TWO3:
    sort_two3( size ); break;
  case TWO31:
    sort_two31( size ); break;
  case TWO32:
    sort_two32( size ); break;
  }
  /* o */
  for( i = 0; i < size; i++ ){
    Uchar *ptr = index_table[i];
    if( ptr == buffer ) top = i;
    work_buffer[i] = *(ptr + size - 1);
  }
  return top;
}

/* ubN\[g  */
void blocksort_decode( Uchar *buff, int top, int size, FILE *out )
{
  Uchar *ptr;
  int i, count[CODE_SIZE];
  /* \[g */
  for( i = 0; i < CODE_SIZE; i++ ) count[i] = 0;
  for( i = 0; i < size; i++ ) count[ buff[i] ]++;
  for( i = 1; i < CODE_SIZE; i++ ) count[i] += count[i - 1];
  for( i = size - 1; i >= 0; i-- ){
    ptr = buff + i;
    index_table[ --count[ *ptr ] ]= ptr;
  }
  /* o */
  ptr = index_table[top]; 
  for( i = 0; i < size; i++ ){
    fputc( *ptr, out );
    ptr = index_table[ptr - buff];
  }
}

/*****  *****/

/* f[^o */
void output_data( Uchar *buff, int top, int size, FILE *out )
{
  /* ʒuƃTCY 3 byte  OK */
  fputc( (top >> 16) & 0xff, out );
  fputc( (top >> 8) & 0xff, out );
  fputc( top & 0xff, out );
  fputc( (size >> 16) & 0xff, out );
  fputc( (size >> 8) & 0xff, out );
  fputc( size & 0xff, out );
  fwrite( buff, 1, size, out );
}

/*  */
void encode( FILE *in, FILE *out )
{
  int top, size;
  /* wb_[Cg */
  fwrite( file_header, 1, HEADER_SIZE, out );
  while( (size = fread( buffer, 1, BUFF_SIZE, in )) > 0 ){
    /* buffer -> work_buffer */
    top = blocksort_encode( size );
    output_data( work_buffer, top, size, out );
  }
}

/* t@C̈k */
void encode_file( void )
{
  FILE	*in, *out;
  /* t@CI[v */
  in = fopen( input_name, "rb" );
  if( in == NULL ){
    fprintf( stderr, "%s ܂\n", input_name );
    exit( 1 );
  }
  out = fopen( output_name, "wb" );
  if( out == NULL ){
    fprintf( stderr, "%s I[vł܂\n", output_name );
    exit( 1 );
  }
  
  encode( in, out );
  fclose( in );
  fclose( out );
}

/*****  *****/

/*  */
void decode( FILE *in, FILE *out )
{
  Uchar *ptr;
  int top, size;
  do {
    /* ʒuƃTCY擾 */
    top = fgetc( in );
    if( top == EOF ) break;
    top = (top << 8) + fgetc( in );
    top = (top << 8) + fgetc( in );
    size = fgetc( in );
    size = (size << 8) + fgetc( in );
    size = (size << 8) + fgetc( in );
    /* f[^[h */
    if( fread( buffer, 1, size, in ) != (size_t)size ){
      fprintf(stderr, "t@CĂ܂ %s\n", input_name );
      exit( 1 );
    }
    blocksort_decode( buffer, top, size, out );
  } while( size == BUFF_SIZE );
}

/* t@C̉ */
void decode_file( void )
{
  FILE *in, *out;
  char buffer[HEADER_SIZE];
  int  size;

  /* ̓t@CI[v */
  in = fopen( input_name, "rb" );
  if( in == NULL ){
    fprintf( stderr, "%s ܂\n", input_name );
    exit( 1 );
  }

  /* wb_[`FbN */
  size = fread( buffer, 1, HEADER_SIZE, in );
  if( size != HEADER_SIZE || memcmp( file_header, buffer, HEADER_SIZE ) ){
    fprintf( stderr, "bs ł͏łȂt@Cł %s\n", input_name );
    exit( 1 );
  }

  /* o̓t@CI[v */
  out = fopen( output_name, "wb" );
  if( out == NULL ){
    fprintf( stderr, "%s I[vł܂\n", output_name );
    exit( 1 );
  }

  decode( in, out );
  fclose( in );
  fclose( out );
}


/***** ͂ƃC *****/

/* gp@ */
void usage( void )
{
  printf("usage : bs [options] input_file output_file\n");
  printf("  -d : , -en : k\n");
  printf("   n = 1: merge sort\n");
  printf("       2: multikey quick sort\n");
  printf("       3: two-stage sort + merge sort (ratio-1)\n");
  printf("       4: two-stage sort + multikey quick sort (ratio-1)\n");
  printf("       5: two-stage sort (ratio-2)\n");
  printf("       6: O@ (ratio-1)\n");
  printf("       7: O@ (ratio-1)\n");
  printf("       8: O@ (ratio-2)\n");
  exit( 1 );
}

/*  */
void getargs( int argc, char *argv[] )
{
  int i;
  for( i = 1; i < argc && argv[i][0] == '-'; i++ ){
    int opt = toupper( argv[i][1] );
    switch( opt ){
    case 'D': dflag = TRUE; break;
    case 'E':
      eflag = atoi( &(argv[i][2]) );
      if( eflag < MERGE || eflag > TWO32 ){
        fprintf( stderr, " -e IvVɂ͐l (1 - %d) w肵Ă\n", TWO32 );
        exit( 1 );
      }
      break;
    case '?': usage(); break;
    default :
      fprintf( stderr, "sȃIvVł\n" );
      usage();
    }
  }
  if( dflag && eflag ){
    fprintf( stderr, "-d  -e ͓Ɏwł܂\n" );
    exit( 1 );
  }
  if( !dflag && !eflag ){
    fprintf( stderr, "-d  -e ̂ǂ炩w肵Ă\n" );
    exit( 1 );
  }
  if( i < argc ){
    strcpy( input_name, argv[i++] );
  } else {
    fprintf( stderr, "̓t@C܂\n");
    exit( 1 );
  }
  if( i < argc ){
    strcpy( output_name, argv[i] );
  } else {
    fprintf( stderr, "o̓t@C܂\n");
    exit( 1 );
  }
}

int main( int argc, char *argv[] )
{
  int start;
  getargs( argc, argv );
  start = clock();
  if( dflag ) decode_file();
  else        encode_file();
  printf(" %d, \[g %d\n", clock() - start, sort_count );
  return 0;
}

